﻿Imports System.Data

''' <summary>
''' DBアクセスクラス。
''' </summary>
''' <remarks></remarks>
Public MustInherit Class DBAccessor

    ''' <summary>
    ''' コネクション
    ''' </summary>
    ''' <remarks></remarks>
    Protected conn As IDbConnection

    ''' <summary>
    ''' トランザクション
    ''' </summary>
    ''' <remarks></remarks>
    Protected transaction As IDbTransaction

    ''' <summary>
    ''' DBと接続する。
    ''' </summary>
    ''' <remarks></remarks>
    Public MustOverride Sub connect()

    ''' <summary>
    ''' DBと接続する。
    ''' </summary>
    ''' <param name="connStr">接続文字列</param>
    ''' <remarks></remarks>
    Public MustOverride Sub connect(ByVal connStr As String)

    ''' <summary>
    ''' トランザクションを開始する
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub beginTransaction()

        If transaction Is Nothing Then
            If conn IsNot Nothing Then
                transaction = conn.BeginTransaction()
            End If
        End If

    End Sub

    ''' <summary>
    ''' コミットする。
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub commit()

        If transaction IsNot Nothing Then

            If conn.State <> ConnectionState.Broken AndAlso _
                conn.State <> ConnectionState.Closed Then

                transaction.Commit()
                transaction.Dispose()
            End If

            transaction = Nothing
        End If
    End Sub

    ''' <summary>
    ''' ロールバックする。
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub rollback()

        If transaction IsNot Nothing Then

            If conn.State <> ConnectionState.Broken AndAlso _
                conn.State <> ConnectionState.Closed Then

                transaction.Rollback()
                transaction.Dispose()
            End If

            transaction = Nothing
        End If
    End Sub

    ''' <summary>
    ''' DBから切断する。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub disconnect()

        ' ロールバックを試みる（事前にコミットしている場合は、このブロックは無効になる）
        rollback()

        If conn IsNot Nothing Then

            If conn.State <> ConnectionState.Broken AndAlso _
                conn.State <> ConnectionState.Closed Then

                conn.Close()
                conn.Dispose()

            End If

            conn = Nothing
        End If

    End Sub

    ''' <summary>
    ''' DBの接続状況を確認する。
    ''' </summary>
    ''' <returns>trueの場合、接続中</returns>
    ''' <remarks></remarks>
    Public Overridable Function isConnect() As Boolean

        If conn IsNot Nothing Then

            If conn.State = ConnectionState.Closed OrElse conn.State = ConnectionState.Broken Then

                Return False

            Else

                Return True
            End If

        Else

            Return False
        End If

    End Function

    ''' <summary>
    ''' リーダーをクローズする。
    ''' </summary>
    ''' <param name="reader">リーダー</param>
    ''' <remarks></remarks>
    Public Sub closeReader(ByVal reader As IDataReader)

        If reader IsNot Nothing Then
            reader.Close()
            reader.Dispose()
        End If

    End Sub

    ''' <summary>
    ''' パラメータをリセットする。
    ''' </summary>
    ''' <remarks></remarks>
    Public MustOverride Sub resetParameters()

    ''' <summary>
    ''' パラメータを追加する。
    ''' </summary>
    ''' <param name="name"></param>
    ''' <param name="value"></param>
    ''' <remarks></remarks>
    Public MustOverride Sub addParameter(ByVal name As String, ByVal value As Object)

    ''' <summary>
    ''' パラメータを追加する。
    ''' </summary>
    ''' <param name="name"></param>
    ''' <param name="t"></param>
    ''' <param name="value"></param>
    ''' <remarks></remarks>
    Public MustOverride Sub addParameter(ByVal name As String, ByVal t As DbType, ByVal value As Object)

    ''' <summary>
    ''' クエリを実行する。
    ''' </summary>
    ''' <param name="sql">SQL</param>
    ''' <returns>データリーダ</returns>
    ''' <remarks></remarks>
    Public MustOverride Function executeQuery( _
                                              ByVal sql As String) As IDataReader

    ''' <summary>
    ''' クエリを実行する。
    ''' </summary>
    ''' <param name="sql">SQL</param>
    ''' <param name="params">パラメータコレクション</param>
    ''' <returns>データセット</returns>
    ''' <remarks></remarks>
    Public MustOverride Function executeQueryDataset( _
                                              ByVal sql As String _
                                            , Optional ByVal params As IDataParameterCollection = Nothing) As DataSet

    ''' <summary>
    ''' データを更新する。
    ''' </summary>
    ''' <param name="sql">SQL</param>
    ''' <returns>更新件数</returns>
    ''' <remarks></remarks>
    Public MustOverride Function executeUpdate(ByVal sql As String) As Integer

End Class
