﻿Imports MySql.Data
Imports MySql.Data.MySqlClient

''' <summary>
''' MySQLアクセッサー
''' </summary>
''' <remarks></remarks>
Public Class MySQLAccessor
    Inherits DBAccessor

    ''' <summary>
    ''' 接続文字列（configから取得する）
    ''' </summary>
    ''' <remarks></remarks>
    Private Shared connStr As String = ConfigurationManager.ConnectionStrings.Item("getme").ConnectionString

    ''' <summary>
    ''' パラメータ
    ''' </summary>
    ''' <remarks></remarks>
    Private paramList As IList(Of MySql.Data.MySqlClient.MySqlParameter)

    ''' <summary>
    ''' データリーダー
    ''' </summary>
    ''' <remarks></remarks>
    Private dataReaderList As IList(Of IDataReader)

    ''' <summary>
    ''' DBと接続する。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overrides Sub connect()

        connect(connStr)
    End Sub

    ''' <summary>
    ''' DBと接続する。
    ''' </summary>
    ''' <param name="connStr">接続文字列</param>
    ''' <remarks></remarks>
    Public Overrides Sub connect(ByVal connStr As String)

        ' コネクションを切断する
        disconnect()

        ' コネクトする
        conn = New MySqlConnection(connStr)
        conn.Open()

        ' トランザクションを開始しておく
        beginTransaction()

        paramList = New List(Of MySql.Data.MySqlClient.MySqlParameter)
        dataReaderList = New List(Of IDataReader)
    End Sub


    ''' <summary>
    ''' DBから切断する。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overrides Sub disconnect()

        If dataReaderList IsNot Nothing Then

            ' パラメータを追加する
            For Each var As IDataReader In dataReaderList

                If Not var.IsClosed Then

                    Me.closeReader(var)
                End If

            Next

            dataReaderList.Clear()
        End If

        MyBase.disconnect()

    End Sub

    ''' <summary>
    ''' パラメータをリセットする。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overrides Sub resetParameters()

        paramList.Clear()
    End Sub

    ''' <summary>
    ''' パラメータを追加する。
    ''' </summary>
    ''' <param name="name"></param>
    ''' <param name="value"></param>
    ''' <remarks></remarks>
    Public Overrides Sub addParameter(ByVal name As String, ByVal value As Object)

        Dim p As New MySql.Data.MySqlClient.MySqlParameter(name, value)
        paramList.Add(p)

    End Sub

    ''' <summary>
    ''' パラメータを追加する。
    ''' </summary>
    ''' <param name="name"></param>
    ''' <param name="t"></param>
    ''' <param name="value"></param>
    ''' <remarks></remarks>
    Public Overrides Sub addParameter(ByVal name As String, ByVal t As DbType, ByVal value As Object)

        Dim p As New MySql.Data.MySqlClient.MySqlParameter
        p.ParameterName = name
        p.DbType = t
        p.Value = value

        paramList.Add(p)

    End Sub

    ''' <summary>
    ''' クエリを実行する。
    ''' </summary>
    ''' <param name="sql">SQL</param>
    ''' <returns>データリーダ</returns>
    ''' <remarks></remarks>
    Public Overrides Function executeQuery( _
                                              ByVal sql As String) As IDataReader

        ' コマンドを生成する
        Dim command As IDbCommand = conn.CreateCommand()
        ' SQLを生成する
        command.CommandText = sql

        ' パラメータを追加する
        For Each param As MySql.Data.MySqlClient.MySqlParameter In paramList
            command.Parameters.Add(param)
        Next

        Dim ret As IDataReader
        ret = command.ExecuteReader()

        dataReaderList.Add(ret)
        ' クエリを実行する
        Return ret

    End Function

    ''' <summary>
    ''' クエリを実行する。
    ''' </summary>
    ''' <param name="sql">SQL</param>
    ''' <param name="params">パラメータコレクション</param>
    ''' <returns>データセット</returns>
    ''' <remarks></remarks>
    Public Overrides Function executeQueryDataset( _
                                              ByVal sql As String _
                                            , Optional ByVal params As IDataParameterCollection = Nothing) As DataSet

        ' 戻り値
        Dim ret As New DataSet

        ' コマンドを生成する
        Dim command As IDbCommand = conn.CreateCommand()
        ' SQLを生成する
        command.CommandText = sql

        For Each param As MySql.Data.MySqlClient.MySqlParameter In paramList
            command.Parameters.Add(param)
        Next

        ' クエリを実行する
        Dim adapter As IDbDataAdapter = New MySqlDataAdapter
        adapter.SelectCommand = command
        adapter.Fill(ret)

        Return ret

    End Function

    ''' <summary>
    ''' データを更新する。
    ''' </summary>
    ''' <param name="sql">SQL</param>
    ''' <returns>更新件数</returns>
    ''' <remarks></remarks>
    Public Overrides Function executeUpdate(ByVal sql As String) As Integer


        ' コマンドを生成する
        Dim command As IDbCommand = conn.CreateCommand()
        ' SQLを生成する
        command.CommandText = sql

        ' パラメータを追加する
        For Each param As MySql.Data.MySqlClient.MySqlParameter In paramList
            command.Parameters.Add(param)
        Next

        ' クエリを実行する
        Return command.ExecuteNonQuery

    End Function

End Class
