﻿Imports System.Reflection

Namespace entity

    Public Class Entity

        ''' <summary>
        ''' 改行コード
        ''' </summary>
        ''' <remarks></remarks>
        Private Const NEW_LINE As String = vbNewLine

        ''' <summary>
        ''' DBAccessorプロパティ（内部メンバ）
        ''' </summary>
        ''' <remarks></remarks>
        Private dbAccessor_ As DBAccessor

        ''' <summary>
        ''' DBAccessorプロパティ
        ''' </summary>
        ''' <value>Id</value>
        ''' <returns>Id</returns>
        ''' <remarks></remarks>
        Public Property DBAccessor() As DBAccessor
            Get
                Return dbAccessor_
            End Get
            Set(ByVal value As DBAccessor)
                dbAccessor_ = value
            End Set
        End Property

        Public Sub New()

        End Sub

        ''' <summary>
        ''' テーブル名を取得する。
        ''' </summary>
        ''' <param name="className">クラス名</param>
        ''' <returns>テーブル名</returns>
        ''' <remarks></remarks>
        Private Function ConvertClassToTableName(ByVal className As String) As String

            Return ConvertCameltoSnake(className)

        End Function

        ''' <summary>
        ''' カラム名を取得する。
        ''' </summary>
        ''' <param name="propertyName">プロパティ名</param>
        ''' <returns>カラム名</returns>
        ''' <remarks></remarks>
        Private Function ConvertPropToColumnName(ByVal propertyName As String) As String

            Return ConvertCameltoSnake(propertyName)

        End Function

        ''' <summary>
        ''' プロパティ名を取得する。
        ''' </summary>
        ''' <param name="columnName">カラム名</param>
        ''' <returns>プロパティ名</returns>
        ''' <remarks></remarks>
        Private Function ConvertColumnToPropName(ByVal columnName As String) As String

            Return ConvertSnaketoCamel(columnName)

        End Function

        ''' <summary>
        ''' キャメルケースからスネークケースに変換する。
        ''' </summary>
        ''' <param name="camelValue">キャメルケース値</param>
        ''' <returns>スネークケース値</returns>
        ''' <remarks></remarks>
        Private Function ConvertCameltoSnake(ByVal camelValue As String) As String

            Dim ret As New StringBuilder(30)

            Dim len As Integer = camelValue.Length

            For i = 0 To len - 1

                If Char.IsUpper(camelValue.Chars(i)) AndAlso i > 0 Then

                    ' 2文字目以降 且つ
                    ' 大文字の場合、先頭にアンダースコアを付与する
                    ret.Append("_" & Char.ToUpper(camelValue.Chars(i)))
                Else

                    ' 小文字の場合、そのまま付与する
                    ret.Append(Char.ToUpper(camelValue.Chars(i)))
                End If

            Next

            Return ret.ToString

        End Function

        ''' <summary>
        ''' スネークケースからキャメルケースに変換する。
        ''' </summary>
        ''' <param name="snakeValue">スネークケース値</param>
        ''' <returns>キャメルケース値</returns>
        ''' <remarks></remarks>
        Private Function ConvertSnaketoCamel(ByVal snakeValue As String) As String

            Dim ret As New StringBuilder(30)

            ' 1文字目は大文字にする
            Dim isUpper As Boolean = True

            Dim len As Integer = snakeValue.Length

            For i = 0 To len - 1

                If snakeValue.Chars(i) = "_" Then

                    ' 大文字の場合、先頭にアンダースコアを付与する
                    isUpper = True
                Else

                    If isUpper Then
                        ' 大文字にする
                        ret.Append(Char.ToUpper(snakeValue.Chars(i)))
                        isUpper = False
                    Else
                        ' 小文字の場合、そのまま付与する
                        ret.Append(Char.ToLower(snakeValue.Chars(i)))

                    End If
                End If

            Next

            Return ret.ToString

        End Function

        ''' <summary>
        ''' Insertを実行する。
        ''' </summary>
        ''' <returns>実行件数</returns>
        ''' <remarks></remarks>
        Public Function ExecInsert() As Integer

            dbAccessor_.resetParameters()

            Dim retCount As Integer

            Dim type As Type

            Dim sql As New StringBuilder
            Dim sqlConjunction As String = " "

            type = Me.GetType()

            ' テーブル名
            Dim tableName As String = ConvertClassToTableName(type.Name)
            ' カラム名
            Dim columnName As String = Nothing

            ' プロパティの一覧を走査する
            Dim properties() As PropertyInfo = type.GetProperties

            ' -----------------------------------------------
            ' テーブル名を出力する
            ' -----------------------------------------------
            sql.Append("INSERT INTO " & tableName & " ( " & NEW_LINE)

            ' -----------------------------------------------
            ' columns句を出力する
            ' -----------------------------------------------
            sqlConjunction = " "
            For Each var As PropertyInfo In properties

                ' 除外するプロパティの場合
                If isExceptProperty(var.Name) Then
                    Continue For
                End If

                If var.CanRead Then

                    ' カラム名を取得する
                    columnName = ConvertPropToColumnName(var.Name)

                    sql.Append("   " & sqlConjunction & columnName & NEW_LINE)
                    sqlConjunction = ","
                End If

            Next

            sql.Append(") VALUES ( " & NEW_LINE)

            ' -----------------------------------------------
            ' values句を出力する
            ' -----------------------------------------------
            sqlConjunction = " "
            For Each var As PropertyInfo In properties

                ' 除外するプロパティの場合
                If isExceptProperty(var.Name) Then
                    Continue For
                End If

                If var.CanRead Then

                    ' カラム名を取得する
                    columnName = ConvertPropToColumnName(var.Name)

                    sql.Append("   " & sqlConjunction & "?" & columnName & NEW_LINE)
                    sqlConjunction = ","

                    dbAccessor_.addParameter(columnName, var.GetGetMethod.Invoke(Me, New Object() {}))
                End If

            Next

            sql.Append(")")

            retCount = dbAccessor_.executeUpdate(sql.ToString)

            Return retCount

        End Function

        ''' <summary>
        ''' Updateを実行する。
        ''' </summary>
        ''' <returns>実行件数</returns>
        ''' <remarks></remarks>
        Public Function ExecUpdate() As Integer

            Return ExecuteUpdate(False)
        End Function

        ''' <summary>
        ''' Updateを実行する。
        ''' </summary>
        ''' <returns>実行件数</returns>
        ''' <remarks></remarks>
        Public Function ExecUpdateNullValueIgnore() As Integer

            Return ExecuteUpdate(True)
        End Function

        ''' <summary>
        ''' Updateを実行する。
        ''' </summary>
        ''' <param name="nullValueIgnore">Null値の場合更新対象外とするかどうか</param>
        ''' <returns>実行件数</returns>
        ''' <remarks></remarks>
        Private Function ExecuteUpdate(ByVal nullValueIgnore As Boolean) As Integer

            dbAccessor_.resetParameters()

            Dim retCount As Integer

            Dim type As Type

            Dim sql As New StringBuilder
            Dim sqlConjunction As String = " "

            type = Me.GetType()

            ' テーブル名
            Dim tableName As String = ConvertClassToTableName(type.Name)
            ' カラム名
            Dim columnName As String = Nothing

            ' プロパティの一覧を走査する
            Dim properties() As PropertyInfo = type.GetProperties

            ' PKリスト
            Dim pkList As New List(Of PropertyInfo)

            ' -----------------------------------------------
            ' テーブル名を出力する
            ' -----------------------------------------------
            sql.Append("UPDATE " & tableName & " SET" & NEW_LINE)

            ' -----------------------------------------------
            ' columns values 句を出力する
            ' -----------------------------------------------
            sqlConjunction = " "
            For Each var As PropertyInfo In properties

                ' 除外するプロパティの場合
                If isExceptProperty(var.Name) Then
                    Continue For
                End If

                If var.CanRead Then

                    ' -------------------------------
                    ' PKの有無を調べる
                    Dim pks() As Object
                    pks = var.GetCustomAttributes(GetType(PrimaryKeyAttribute), True)

                    If pks IsNot Nothing AndAlso _
                       pks.Length > 0 Then

                        pkList.Add(var)

                    Else

                        ' PKではない場合

                        Dim val As Object
                        val = var.GetGetMethod.Invoke(Me, New Object() {})

                        Dim isAdd As Boolean = False

                        If nullValueIgnore Then

                            If val IsNot Nothing Then
                                isAdd = True
                            End If
                        Else
                            isAdd = True
                        End If

                        If isAdd Then

                            ' カラム名を取得する
                            columnName = ConvertPropToColumnName(var.Name)

                            sql.Append("   " & sqlConjunction & columnName & " = ?" & columnName & NEW_LINE)
                            sqlConjunction = ","

                            dbAccessor_.addParameter(columnName, val)

                        End If

                    End If
                    ' -------------------------------

                End If

            Next

            ' -----------------------------------------------
            ' WHERE句を出力する
            ' -----------------------------------------------
            sql.Append("WHERE" & NEW_LINE)
            sqlConjunction = " "
            For Each var As PropertyInfo In pkList

                ' カラム名を取得する
                columnName = ConvertPropToColumnName(var.Name)

                sql.Append("   " & sqlConjunction & columnName & " = ?" & columnName & NEW_LINE)
                sqlConjunction = "AND "

                dbAccessor_.addParameter(columnName, var.GetGetMethod.Invoke(Me, New Object() {}))

            Next

            retCount = dbAccessor_.executeUpdate(sql.ToString)

            Return retCount

        End Function

        ''' <summary>
        ''' 削除を実行する。
        ''' </summary>
        ''' <returns>実行件数</returns>
        ''' <remarks></remarks>
        Public Function ExecDelete() As Integer

            dbAccessor_.resetParameters()

            Dim retCount As Integer

            Dim type As Type

            Dim sql As New StringBuilder
            Dim sqlConjunction As String = " "

            type = Me.GetType()

            ' テーブル名
            Dim tableName As String = ConvertClassToTableName(type.Name)
            ' カラム名
            Dim columnName As String = Nothing

            ' プロパティの一覧を走査する
            Dim properties() As PropertyInfo = type.GetProperties

            ' -----------------------------------------------
            ' テーブル名を出力する
            ' -----------------------------------------------
            sql.Append("DELETE FROM " & tableName & NEW_LINE)

            ' -----------------------------------------------
            ' WHERE句を出力する
            ' -----------------------------------------------
            sql.Append("WHERE" & NEW_LINE)
            sqlConjunction = " "
            For Each var As PropertyInfo In properties

                ' 除外するプロパティの場合
                If isExceptProperty(var.Name) Then
                    Continue For
                End If

                If var.CanRead Then

                    ' -------------------------------
                    ' PKの有無を調べる
                    Dim pks() As Object
                    pks = var.GetCustomAttributes(GetType(PrimaryKeyAttribute), True)

                    If pks IsNot Nothing AndAlso _
                       pks.Length > 0 Then

                        Dim pkConst As PrimaryKeyAttribute
                        pkConst = CType(pks(0), PrimaryKeyAttribute)

                        If pkConst.IsPk Then
                            ' PKの場合

                            ' カラム名を取得する
                            columnName = ConvertPropToColumnName(var.Name)

                            sql.Append("   " & sqlConjunction & columnName & " = ?" & columnName & NEW_LINE)
                            sqlConjunction = "AND "

                            dbAccessor_.addParameter(columnName, var.GetGetMethod.Invoke(Me, New Object() {}))

                        End If

                    End If
                    ' -------------------------------

                End If

            Next

            retCount = dbAccessor_.executeUpdate(sql.ToString)

            Return retCount

        End Function

        ''' <summary>
        ''' Selectを実行する
        ''' </summary>
        ''' <returns>取得件数</returns>
        ''' <remarks></remarks>
        Public Function ExecSelect() As Integer

            dbAccessor_.resetParameters()

            Dim propMap As New Dictionary(Of String, PropertyInfo)

            Dim type As Type

            Dim sql As New StringBuilder
            Dim sqlConjunction As String = " "

            type = Me.GetType()

            ' テーブル名
            Dim tableName As String = ConvertClassToTableName(type.Name)
            ' カラム名
            Dim columnName As String = Nothing

            ' プロパティの一覧を走査する
            Dim properties() As PropertyInfo = type.GetProperties

            ' PKリスト
            Dim pkList As New List(Of PropertyInfo)

            ' -----------------------------------------------
            ' SELECT 句を出力する
            ' -----------------------------------------------
            sql.Append("SELECT" & NEW_LINE)

            sqlConjunction = " "
            For Each var As PropertyInfo In properties

                ' 除外するプロパティの場合
                If isExceptProperty(var.Name) Then
                    Continue For
                End If

                If var.CanRead Then

                    ' -------------------------------
                    ' PKの有無を調べる
                    Dim pks() As Object
                    pks = var.GetCustomAttributes(GetType(PrimaryKeyAttribute), True)

                    If pks IsNot Nothing AndAlso _
                       pks.Length > 0 Then

                        Dim pkConst As PrimaryKeyAttribute
                        pkConst = CType(pks(0), PrimaryKeyAttribute)

                        If pkConst.IsPk Then
                            ' PKの場合
                            pkList.Add(var)

                        End If

                    End If
                    ' -------------------------------

                    Dim val As Object
                    val = var.GetGetMethod.Invoke(Me, New Object() {})

                    ' カラム名を取得する
                    columnName = ConvertPropToColumnName(var.Name)

                    sql.Append("   " & sqlConjunction & columnName & NEW_LINE)
                    sqlConjunction = ","

                    propMap.Add(var.Name, var)
                End If

            Next
            ' -----------------------------------------------
            ' テーブル名を出力する
            ' -----------------------------------------------
            sql.Append("FROM " & tableName & NEW_LINE)


            sql.Append("WHERE" & NEW_LINE)

            ' -----------------------------------------------
            ' WHERE句を出力する
            ' -----------------------------------------------
            sqlConjunction = "    "
            For Each var As PropertyInfo In pkList

                ' カラム名を取得する
                columnName = ConvertPropToColumnName(var.Name)

                sql.Append("   " & sqlConjunction & columnName & " = ?" & columnName & NEW_LINE)
                sqlConjunction = "AND "

                dbAccessor_.addParameter(columnName, var.GetGetMethod.Invoke(Me, New Object() {}))

            Next

            Dim r As IDataReader = Nothing

            Try
                r = dbAccessor_.executeQuery(sql.ToString)

                If r.Read Then

                    Dim cnt As Integer
                    cnt = r.FieldCount

                    For i = 0 To cnt - 1

                        Dim name As String
                        Dim val As Object

                        name = r.GetName(i)
                        val = r.GetValue(i)

                        name = ConvertColumnToPropName(name)

                        If propMap.ContainsKey(name) Then

                            Dim prop As PropertyInfo
                            prop = propMap.Item(name)

                            If prop.CanWrite Then

                                prop.GetSetMethod.Invoke(Me, New Object() {Util.convertTo(prop.PropertyType, val)})
                            End If

                        End If

                    Next

                    Return 1

                End If

            Finally

                dbAccessor_.closeReader(r)

            End Try

            Return 0

        End Function

        ''' <summary>
        ''' 除外対象プロパティの判定。
        ''' </summary>
        ''' <param name="propertyName">プロパティ名</param>
        ''' <returns>trueの場合、除外対象</returns>
        ''' <remarks></remarks>
        Private Function isExceptProperty(ByVal propertyName As String) As Boolean

            If propertyName = "DBAccessor" Then

                isExceptProperty = True

            Else

                isExceptProperty = False
            End If

        End Function

    End Class

End Namespace
