DROP DATABASE IF EXISTS getme;

CREATE DATABASE getme
    CHARACTER SET = 'utf8'
    COLLATE       = 'utf8_general_ci'
;

DROP USER getme@localhost;
CREATE USER getme@localhost IDENTIFIED BY 'getme';
GRANT ALL ON getme.* TO getme@localhost;

FLUSH PRIVILEGES;

USE GETME;

DROP TABLE IF EXISTS ITEM;
CREATE TABLE ITEM(
      ID                        SERIAL             NOT NULL                     COMMENT 'ID'
    , ITEM_NAME                 VARCHAR (30)       NOT NULL                     COMMENT '商品名'
    , PRICE                     INT                                             COMMENT '金額'
    , IMAGE                     MEDIUMBLOB                                      COMMENT '画像'
    , REGIST_DATE               DATETIME           NOT NULL                     COMMENT '登録年月日'
    , REGIST_USER_ID            BIGINT UNSIGNED    NOT NULL                     COMMENT '登録ユーザID'
    , UPDATE_DATE               DATETIME           NOT NULL                     COMMENT '更新年月日'
    , UPDATE_USER_ID            BIGINT UNSIGNED    NOT NULL                     COMMENT '更新ユーザID'
    , VERSION                   BIGINT UNSIGNED    NOT NULL                     COMMENT 'バージョン'
    , PRIMARY KEY (
          ID
    )
) ENGINE = INNODB COMMENT = '商品';

DROP TABLE IF EXISTS ORDERS;
CREATE TABLE ORDERS (
      ID                        SERIAL             NOT NULL                     COMMENT 'ID'
    , NAME_FIRST                VARCHAR (30)       NOT NULL                     COMMENT '氏'
    , NAME_LAST                 VARCHAR (30)       NOT NULL                     COMMENT '名'
    , NAME_FIRST_KANA           VARCHAR (30)       NOT NULL                     COMMENT '氏（カナ）'
    , NAME_LAST_KANA            VARCHAR (30)       NOT NULL                     COMMENT '名（カナ）'
    , ZIP_CODE                  VARCHAR (7)        NOT NULL                     COMMENT '郵便番号'
    , REGION                    VARCHAR (30)       NOT NULL                     COMMENT '都道府県'
    , CITY                      VARCHAR (30)       NOT NULL                     COMMENT '市区町村'
    , ADDRESS1                  VARCHAR (50)       NOT NULL                     COMMENT '住所１'
    , ADDRESS2                  VARCHAR (50)       NOT NULL                     COMMENT '住所２（方書）'
    , TEL1                      VARCHAR (15)       NOT NULL                     COMMENT '電話番号'
    , TEL2                      VARCHAR (15)       NOT NULL                     COMMENT '電話番号（携帯電話）'
    , REGIST_DATE               DATETIME           NOT NULL                     COMMENT '登録年月日'
    , REGIST_USER_ID            BIGINT UNSIGNED    NOT NULL                     COMMENT '登録ユーザID'
    , UPDATE_DATE               DATETIME           NOT NULL                     COMMENT '更新年月日'
    , UPDATE_USER_ID            BIGINT UNSIGNED    NOT NULL                     COMMENT '更新ユーザID'
    , VERSION                   BIGINT UNSIGNED    NOT NULL                     COMMENT 'バージョン'
    , PRIMARY KEY (
          ID
    )
) ENGINE = INNODB COMMENT = '受注';

DROP TABLE IF EXISTS ORDERS_DETAIL;
CREATE TABLE ORDERS_DETAIL (
      ID                        SERIAL             NOT NULL                     COMMENT 'ID'
    , ORDER_ID                  BIGINT UNSIGNED    NOT NULL                     COMMENT '受注ID'
    , ITEM_NAME                 VARCHAR (30)       NOT NULL                     COMMENT '商品名'
    , QUANTITY                  INT  (5)                                        COMMENT '数量'
    , REGIST_DATE               DATETIME           NOT NULL                     COMMENT '登録年月日'
    , REGIST_USER_ID            BIGINT UNSIGNED    NOT NULL                     COMMENT '登録ユーザID'
    , UPDATE_DATE               DATETIME           NOT NULL                     COMMENT '更新年月日'
    , UPDATE_USER_ID            BIGINT UNSIGNED    NOT NULL                     COMMENT '更新ユーザID'
    , VERSION                   BIGINT UNSIGNED    NOT NULL                     COMMENT 'バージョン'
    , PRIMARY KEY (
          ID
    )
) ENGINE = INNODB COMMENT = '受注明細';
